/*
 * ******************************************************************************************
 *
 * File		: apb_spi.h
 * Author	: GOWIN
 * Function	: Definition of APB SPI
 *
 * ******************************************************************************************
 */

#ifndef __APB_SPI_H__
#define __APB_SPI_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include <stddef.h>


// Definitions ------------------------------------------------------------------------------

/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB SPI (Serial Peripheral Interface)
 ****************************************************************************/
typedef struct
{
	__RO unsigned int IDREV;                /* 0x00 ID and Revision Register           */
	     unsigned int RESERVED0[3];         /* 0x04~0x0C Reserved                      */
	__RW unsigned int TRANSFMT;             /* 0x10 SPI Transfer Format Register       */
	__RW unsigned int DIRECTIO;             /* 0x14 SPI Direct IO Control Register     */
	     unsigned int RESERVED1[2];         /* 0x18~0x1C Reserved                      */
	__RW unsigned int TRANSCTRL;            /* 0x20 SPI Transfer Control Register      */
	__RW unsigned int CMD;                  /* 0x24 SPI Command Register               */
	__RW unsigned int ADDR;                 /* 0x28 SPI Address Register               */
	__RW unsigned int DATA;                 /* 0x2C SPI Data Register                  */
	__RW unsigned int CTRL;                 /* 0x30 SPI Control Register               */
	__RO unsigned int STATUS;               /* 0x34 SPI Status Register                */
	__RW unsigned int INTREN;               /* 0x38 SPI Interrupt Enable Register      */
	__WO unsigned int INTRST;               /* 0x3C SPI Interrupt Status Register      */
	__RW unsigned int TIMING;               /* 0x40 SPI Interface Timing Register      */
	     unsigned int RESERVED2[3];         /* 0x44~0x4C Reserved                      */
	__RW unsigned int MEMCTRL;              /* 0x50 SPI Memory Access Control Register */
	     unsigned int RESERVED3[3];         /* 0x54 ~ 0x5C Reserved                    */
	__RW unsigned int SLVST;                /* 0x60 SPI Slave Status Register          */
	__RO unsigned int SLVDATACNT;           /* 0x64 SPI Slave Data Count Register      */
	     unsigned int RESERVED4[5];         /* 0x68~0x78 Reserved                      */
	__RO unsigned int CONFIG;               /* 0x7C Configuration Register             */
} APB_SPI_RegDef;


/* General return codes */
#define APB_SPI_DRIVER_OK                 0 // Operation succeeded
#define APB_SPI_DRIVER_ERROR             -1 // Unspecified error
#define APB_SPI_DRIVER_ERROR_BUSY        -2 // Driver is busy
#define APB_SPI_DRIVER_ERROR_TIMEOUT     -3 // Timeout occurred
#define APB_SPI_DRIVER_ERROR_UNSUPPORTED -4 // Operation not supported
#define APB_SPI_DRIVER_ERROR_PARAMETER   -5 // Parameter error
#define APB_SPI_DRIVER_ERROR_SPECIFIC    -6 // Start of driver specific errors


#ifdef __cplusplus
}
#endif


#endif	/* __APB_SPI_H__ */
