/*
 * ******************************************************************************************
 *
 * File		: apb_spi_flash_driver.h
 * Author	: GOWIN
 * Function	: Driver definition of APB SPI Flash
 *
 * ******************************************************************************************
 */

#ifndef __APB_SPI_FLASH_DRIVER_H__
#define __APB_SPI_FLASH_DRIVER_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_spi.h"


// Definitions ------------------------------------------------------------------------------

// Flash command definitions
#define APB_SPI_FLASH_WRITE_CMD				0x01	// Write
#define	APB_SPI_FLASH_PROGRAM_CMD			0x02	// Page program
#define APB_SPI_FLASH_READ_CMD				0x03	// Read
#define APB_SPI_FLASH_WRITE_DISABLE_CMD   	0x04	// Disable write
#define APB_SPI_FLASH_READ_STATUS_CMD		0x05	// Read status
#define APB_SPI_FLASH_WRITE_ENABLE_CMD    	0x06	// Enable write
#define	APB_SPI_FLASH_ERASE_4K_CMD			0x20	// Erase 4k
#define APB_SPI_FLASH_ERASE_64K_CMD			0xD8	// Erase 64k
#define APB_SPI_FLASH_ERASE_CHIP_CMD      	0x60  	// Erase full chip


// ** Functions ** //

// Initializes APB SPI Flash
int apb_spi_flash_init(APB_SPI_RegDef *DEV_FLASH);

// Switch APB SPI Flash mode between download and read/write/erase memory
int apb_spi_flash_switch_mode(APB_SPI_RegDef *DEV_FLASH);

// Read APB SPI Flash data
int apb_spi_flash_read(APB_SPI_RegDef *DEV_FLASH, unsigned int len, unsigned int cmd, unsigned int address, unsigned char *buf);

// Write APB SPI Flash data
int apb_spi_flash_write(APB_SPI_RegDef *DEV_FLASH, unsigned int len, unsigned int cmd, unsigned int address, unsigned char *buf);

// Program APB SPI Flash data
int apb_spi_flash_program(APB_SPI_RegDef *DEV_FLASH, unsigned int len, unsigned int address, unsigned char *buf);

// Erase 4K sector
int apb_spi_flash_erase_4ksec(APB_SPI_RegDef *DEV_FLASH, unsigned int address);

// Erase 64K sector
int apb_spi_flash_erase_64ksec(APB_SPI_RegDef *DEV_FLASH, unsigned int address);


#ifdef __cplusplus
}
#endif


#endif	/* __APB_SPI_FLASH_DRIVER_H__ */
