
// apb_spi_flash_mem_demo
module apb_spi_flash_mem_demo
(
    input CLK,
    input RSTN,
    // UART2
    output UART2_TXD,
    input  UART2_RXD,
    // SPI Flash Memory
    inout FLASH_SPI_CSN,
    inout FLASH_SPI_MISO,
    inout FLASH_SPI_MOSI,
    inout FLASH_SPI_CLK,
    inout FLASH_SPI_HOLDN,
    inout FLASH_SPI_WPN,
    // DDR3 Memory
    output DDR3_INIT,
    output [2:0] DDR3_BANK,
    output DDR3_CS_N,
    output DDR3_RAS_N,
    output DDR3_CAS_N,
    output DDR3_WE_N,
    output DDR3_CK,
    output DDR3_CK_N,
    output DDR3_CKE,
    output DDR3_RESET_N,
    output DDR3_ODT,
    output [13:0] DDR3_ADDR,
    output [1:0] DDR3_DM,
    inout [15:0] DDR3_DQ,
    inout [1:0] DDR3_DQS,
    inout [1:0] DDR3_DQS_N,
    // JTAG
    input TCK_IN,
    input TMS_IN,
    input TRST_IN,
    input TDI_IN,
    output TDO_OUT
);


wire CORE_CLK;
wire DDR_CLK;
wire AHB_CLK;
wire APB_CLK;
wire RTC_CLK;

wire DDR3_MEMORY_CLK;
wire DDR3_CLK_IN;
wire DDR3_LOCK;
wire DDR3_STOP;


// Gowin_PLL_AE350 instantiation
Gowin_PLL_AE350 u_Gowin_PLL_AE350
(
    .clkout0(DDR_CLK),          // 50MHz    - for test
    .clkout1(CORE_CLK),         // 800MHz
    .clkout2(AHB_CLK),          // 50MHz    - for test
    .clkout3(APB_CLK),          // 50MHz    - for test
    .clkout4(RTC_CLK),          // 10MHz
    .clkin(CLK),
    .init_clk(CLK),
    .reset(1'b0),
    .enclk0(1'b1),
    .enclk1(1'b1),
    .enclk2(1'b1),
    .enclk3(1'b1),
    .enclk4(1'b1)
);


// Gowin_PLL_DDR3 instantiation
Gowin_PLL_DDR3 u_Gowin_PLL_DDR3
(
    .lock(DDR3_LOCK),
    .clkout0(DDR3_CLK_IN),          // 50MHz
    .clkout2(DDR3_MEMORY_CLK),      // 200MHz   - for test
    .clkin(CLK),
    .init_clk(CLK),
    .reset(1'b0),
    .enclk0(1'b1),
    .enclk2(DDR3_STOP)
);


wire ae350_rstn;                    // AE350 power on and hardware reset in
wire ddr3_rstn;                     // DDR3 memory reset in
wire ddr3_init_completed;           // DDR3 memory initialized completed

assign DDR3_INIT = ddr3_init_completed;


// key_debounce instantiation
// DDR3 memory reset in key debounce
key_debounce u_key_debounce_ddr3
(
    .out(ddr3_rstn),
    .in(RSTN),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// key_debounce instantiation
// AE350 power on and hardware reset in key debounce
key_debounce u_key_debounce_ae350
(
    .out(ae350_rstn),
    .in(ddr3_init_completed),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// APB bus
wire [31:0] APB_PRDATA;
wire        APB_PREADY;
wire [31:0] APB_PADDR;
wire        APB_PENABLE;
wire        APB_PSEL;
wire [31:0] APB_PWDATA;
wire        APB_PWRITE;
wire [2:0]  APB_PPROT;
wire [3:0]  APB_PSTRB;
wire        APB_PCLK;
wire        APB_PRSTN;

// APB SPI DMA signals
wire spi_rx_dma_ack;
wire spi_rx_dma_req;
wire spi_tx_dma_ack;
wire spi_tx_dma_req;

wire [5:0] dma_req;
wire [5:0] dma_ack;
assign dma_req = 6'b0;

// APB SPI interrupt
wire spi_intr;

// APB SPI signals
wire spi_hold_n_in;
wire spi_hold_n_oe;
wire spi_hold_n_out;
wire spi_wp_n_in;
wire spi_wp_n_oe;
wire spi_wp_n_out;
wire spi_cs_n_in;
wire spi_cs_n_oe;
wire spi_cs_n_out;
wire spi_clk_in;
wire spi_clk_oe;
wire spi_clk_out;
wire spi_miso_in;
wire spi_miso_oe;
wire spi_miso_out;
wire spi_mosi_in;
wire spi_mosi_oe;
wire spi_mosi_out;

// APB SPI inout port
gw_inout_io spi_hold_n_pad (.O(spi_hold_n_in), .I(spi_hold_n_out), .IO(FLASH_SPI_HOLDN), .E(spi_hold_n_oe), .PU(1'b0), .PD(1'b0));
gw_inout_io spi_wp_n_pad   (.O(spi_wp_n_in),   .I(spi_wp_n_out),   .IO(FLASH_SPI_WPN),   .E(spi_wp_n_oe),   .PU(1'b0), .PD(1'b0));
gw_inout_io spi_clk_pad	   (.O(spi_clk_in),    .I(spi_clk_out),    .IO(FLASH_SPI_CLK),   .E(spi_clk_oe),    .PU(1'b0), .PD(1'b0));
gw_inout_io spi_cs_n_pad   (.O(spi_cs_n_in),   .I(spi_cs_n_out),   .IO(FLASH_SPI_CSN),   .E(spi_cs_n_oe),   .PU(1'b0), .PD(1'b0));
gw_inout_io spi_miso_pad   (.O(spi_miso_in),   .I(spi_miso_out),   .IO(FLASH_SPI_MISO),  .E(spi_miso_oe),   .PU(1'b0), .PD(1'b0));
gw_inout_io spi_mosi_pad   (.O(spi_mosi_in),   .I(spi_mosi_out),   .IO(FLASH_SPI_MOSI),  .E(spi_mosi_oe),   .PU(1'b0), .PD(1'b0));


// ROM AHB bus
wire [31:0] ROM_HRDATA;
wire ROM_HREADY;       // input
wire ROM_HRESP;
wire [31:0] ROM_HADDR;
wire [1:0] ROM_HTRANS;
wire ROM_HWRITE;
wire ROM_HCLK;
wire ROM_HRSTN;

wire hresp;


// APB_SPI_Top instantiation
APB_SPI_Top u_APB_SPI_Top
(
  .hclk(ROM_HCLK),
  .hresetn(ROM_HRSTN),
  .pclk(APB_PCLK),
  .presetn(APB_PRSTN),
  .paddr(APB_PADDR),
  .penable(APB_PENABLE),
  .prdata(APB_PRDATA),
  .pready(APB_PREADY),
  .psel(APB_PSEL),
  .pwdata(APB_PWDATA),
  .pwrite(APB_PWRITE),
  .haddr_mem(ROM_HADDR),
  .hrdata_mem(ROM_HRDATA),
  .hreadyin_mem(ROM_HREADY),
  .hreadyout_mem(ROM_HREADY),
  .hresp_mem({hresp, ROM_HRESP}),
  .hsel_mem(1'b1),
  .htrans_mem(ROM_HTRANS),
  .hwrite_mem(ROM_HWRITE),
  .spi_default_as_slave(1'b0),
  .spi_cs_n_in(spi_cs_n_in),
  .spi_hold_n_in(spi_hold_n_in),
  .spi_hold_n_oe(spi_hold_n_oe),
  .spi_hold_n_out(spi_hold_n_out),
  .spi_wp_n_in(spi_wp_n_in),
  .spi_wp_n_oe(spi_wp_n_oe),
  .spi_wp_n_out(spi_wp_n_out),
  .apb2ahb_clken(1'b1),
  .spi_clock(APB_PCLK),
  .spi_rstn(APB_PRSTN),
  .spi_boot_intr(spi_intr),
  .spi_default_mode3(1'b0),
  .spi_rx_dma_ack(spi_rx_dma_ack),
  .spi_rx_dma_req(spi_rx_dma_req),
  .spi_tx_dma_ack(spi_tx_dma_ack),
  .spi_tx_dma_req(spi_tx_dma_req),
  .scan_enable(1'b0),
  .scan_test(1'b0),
  .spi_clk_in(spi_clk_in),
  .spi_clk_oe(spi_clk_oe),
  .spi_clk_out(spi_clk_out),
  .spi_cs_n_oe(spi_cs_n_oe),
  .spi_cs_n_out(spi_cs_n_out),
  .spi_miso_in(spi_miso_in),
  .spi_miso_oe(spi_miso_oe),
  .spi_miso_out(spi_miso_out),
  .spi_mosi_in(spi_mosi_in),
  .spi_mosi_oe(spi_mosi_oe),
  .spi_mosi_out(spi_mosi_out)
);


// RiscV_AE350_SOC_Top instantiation
RiscV_AE350_SOC_Top u_RiscV_AE350_SOC_Top
(
    .ROM_HRDATA(ROM_HRDATA),
    .ROM_HREADY(ROM_HREADY),
    .ROM_HRESP(ROM_HRESP),
    .ROM_HADDR(ROM_HADDR),
    .ROM_HTRANS(ROM_HTRANS),
    .ROM_HWRITE(ROM_HWRITE),
    .ROM_HCLK(ROM_HCLK),
    .ROM_HRSTN(ROM_HRSTN),
    .DDR3_MEMORY_CLK(DDR3_MEMORY_CLK),
    .DDR3_CLK_IN(DDR3_CLK_IN),
    .DDR3_RSTN(ddr3_rstn),              // DDR3 memory reset in, 0 is reset state
    .DDR3_LOCK(DDR3_LOCK),
    .DDR3_STOP(DDR3_STOP),
    .DDR3_INIT(ddr3_init_completed),    // DDR3 memory initialized completed
    .DDR3_BANK(DDR3_BANK),
    .DDR3_CS_N(DDR3_CS_N),
    .DDR3_RAS_N(DDR3_RAS_N),
    .DDR3_CAS_N(DDR3_CAS_N),
    .DDR3_WE_N(DDR3_WE_N),
    .DDR3_CK(DDR3_CK),
    .DDR3_CK_N(DDR3_CK_N),
    .DDR3_CKE(DDR3_CKE),
    .DDR3_RESET_N(DDR3_RESET_N),
    .DDR3_ODT(DDR3_ODT),
    .DDR3_ADDR(DDR3_ADDR),
    .DDR3_DM(DDR3_DM),
    .DDR3_DQ(DDR3_DQ),
    .DDR3_DQS(DDR3_DQS),
    .DDR3_DQS_N(DDR3_DQS_N),
    .TCK_IN(TCK_IN),
    .TMS_IN(TMS_IN),
    .TRST_IN(TRST_IN),
    .TDI_IN(TDI_IN),
    .TDO_OUT(TDO_OUT),
    .TDO_OE(),
    .EXT_INT({15'b0, spi_intr}),
    .APB_PRDATA(APB_PRDATA),
	.APB_PREADY(APB_PREADY),
	.APB_PSLVERR(1'b0),
	.APB_PADDR(APB_PADDR),
	.APB_PENABLE(APB_PENABLE),
	.APB_PSEL(APB_PSEL),
	.APB_PWDATA(APB_PWDATA),
	.APB_PWRITE(APB_PWRITE),
	.APB_PPROT(APB_PPROT),
	.APB_PSTRB(APB_PSTRB),
	.APB_PCLK(APB_PCLK),
	.APB_PRSTN(APB_PRSTN),
    .DMA_REQ({dma_req, spi_rx_dma_req,spi_tx_dma_req}),
	.DMA_ACK({dma_ack, spi_rx_dma_ack,spi_tx_dma_ack}),
    .UART2_TXD(UART2_TXD),
    .UART2_RTSN(),
    .UART2_RXD(UART2_RXD),
    .UART2_CTSN(1'b0),
    .UART2_DCDN(1'b0),
    .UART2_DSRN(1'b0),
    .UART2_RIN(1'b0),
    .UART2_DTRN(),
    .UART2_OUT1N(),
    .UART2_OUT2N(),
    .CORE_CLK(CORE_CLK),
    .DDR_CLK(DDR_CLK),
    .AHB_CLK(AHB_CLK),
    .APB_CLK(APB_CLK),
    .RTC_CLK(RTC_CLK),
    .POR_RSTN(ae350_rstn),              // AE350 CPU core power on reset, 0 is reset state
    .HW_RSTN(ae350_rstn)                // AE350 hardware reset, 0 is reset state
);

endmodule




// gw_inout_io
/*
module gw_inout_io
(
    O,
    I,
    IO,
    E,
    PU,
    PD
);

output  O;
input   I;
inout   IO;
input   E;
input   PU;
input   PD;

assign  IO	= E ? I : 1'bz;
assign  O	= IO;
assign  (pull0, pull1) IO = PU ? 1'b1 :
                            PD ? 1'b0 : 1'bz;

endmodule
*/

module gw_inout_io
(
  O,
  I,
  IO,
  E,
  PU,
  PD
);

output O;
input I;
inout IO;
input E;
input PU;
input PD;

wire IO_19;
wire IO_18;
wire VCC;
wire GND;

IOBUF IO_iobuf
(
    .O(O),
    .IO(IO),
    .I(IO_19),
    .OEN(IO_18)
);

LUT3 IO_s11
(
    .F(IO_19),
    .I0(E),
    .I1(I),
    .I2(PU)
);
defparam IO_s11.INIT=8'hF8;

LUT3 IO_s12
(
    .F(IO_18),
    .I0(PU),
    .I1(PD),
    .I2(E)
);
defparam IO_s12.INIT=8'h01;

VCC VCC_cZ
(
    .V(VCC)
);

GND GND_cZ
(
    .G(GND)
);

GSR GSR
(
    .GSRI(VCC)
);

endmodule