/*
 * ******************************************************************************************
 *
 * File		: apb_spi_dev.c
 * Author	: GOWIN
 * Function	: Device definition of APB SPI
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_spi_dev.h"


// Definitions  -----------------------------------------------------------------------------

// Get capabilities
APB_SPI_CAPABILITIES apb_spi1_get_capabilities(void)
{
	return apb_spix_get_capabilities(&apb_spi1_resources);
}

// Initializes
int apb_spi1_initialize(APB_SPI_SignalEvent_t cb_event)
{
	return apb_spix_initialize(cb_event, &apb_spi1_resources);
}

// Uninitialized
int apb_spi1_uninitialize (void)
{
	return apb_spix_uninitialize(&apb_spi1_resources);
}

// Power control
int apb_spi1_power_control(APB_SPI_POWER_STATE state)
{
	return apb_spix_power_control(state, &apb_spi1_resources);
}

// Send data
int apb_spi1_send(const void *data, unsigned int num)
{
	return apb_spix_send(data, num, &apb_spi1_resources);
}

// Receive data
int apb_spi1_receive(void *data, unsigned int num)
{
	return apb_spix_receive(data, num, &apb_spi1_resources);
}

// Transfer data
int apb_spi1_transfer(const void *data_out, void *data_in, unsigned int num)
{
	return apb_spix_transfer(data_out, data_in, num, &apb_spi1_resources);
}

// Get data count
unsigned int apb_spi1_get_data_count(void)
{
	return apb_spix_get_data_count(&apb_spi1_resources);
}

// Control command
int apb_spi1_control(unsigned int control, unsigned int arg)
{
	return apb_spix_control(control, arg, &apb_spi1_resources);
}

// Get status
APB_SPI_STATUS apb_spi1_get_status(void)
{
	return apb_spix_get_status(&apb_spi1_resources);
}

#if (APB_SPI1_DMA_EN == 1)
// DMA TX event
void apb_spi1_dma_tx_event (unsigned int event)
{
	apb_spix_dma_tx_event(event, &apb_spi1_resources);
}

// DMA RX event
void apb_spi1_dma_rx_event (unsigned int event)
{
	apb_spix_dma_rx_event(event, &apb_spi1_resources);
}
#endif

// APB SPI interrupt handler
void apb_spi1_irq_handler(void)
{
	apb_spix_irq_handler(&apb_spi1_resources);
}
