/*
 * ******************************************************************************************
 *
 * File		: apb_spi_dev.h
 * Author	: GOWIN
 * Function	: Device definition of APB SPI
 *
 * ******************************************************************************************
 */

#ifndef __APB_SPI_DEV_H__
#define __APB_SPI_DEV_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_spi_driver.h"
#include "platform.h"


// Definitions ------------------------------------------------------------------------------

/*************************************************************************************
 * An APB SPI device "APB SPI1" or "APB_SPI_FLASH" is defined as a user template.
 * User can define more APB SPI or Flash devices.
 *************************************************************************************/

/*************************************************************************************
 * "X" represents user customized.
 *************************************************************************************/

// SPI device
#define APB_SPI1_BASE					APB_EXT_BASE/*X*/						// APB SPI1 address definition
#define APB_SPI1						((APB_SPI_RegDef *) APB_SPI1_BASE )		// APB SPI1 register definition

#define IRQ_APB_SPI1_SOURCE				IRQ_GP0_SOURCE/*X*/						// APB SPI1 interrupt source
#define apb_spi1_irq_handler			gp0_irq_handler/*X*/					// APB SPI1 interrupt handler


// Flash device
#define APB_SPI_FLASH_BASE				APB_EXT_BASE/*X*/						// APB SPI Flash address definition
#define APB_SPI_FLASH					((APB_SPI_RegDef *) APB_SPI_FLASH_BASE )// APB SPI Flash register definition

#define IRQ_APB_SPI_FLASH_SOURCE		IRQ_GP0_SOURCE/*X*/						// APB SPI Flash interrupt source
#define apb_spi_flash_irq_handler		gp0_irq_handler/*X*/					// APB SPI Flash interrupt handler




// APB SPI1 with DMA
#if (APB_SPI_DMA_EN == 0)
#define APB_SPI1_DMA_EN					0
#else
#define APB_SPI1_DMA_EN              	1/*X*/		// Enable/disable APB SPI1 DMA. X: 1: enable; 0: disable.
#endif
// DMA TX
#define APB_SPI1_DMA_TX_CH              0/*X*/		// DMA channel number for APB SPI1 TX
#define APB_SPI1_DMA_TX_REQID           0/*X*/		// DMA request ID for APB SPI1 TX
// DMA RX
#define APB_SPI1_DMA_RX_CH              1/*X*/		// DMA channel number for APB SPI1 RX
#define APB_SPI1_DMA_RX_REQID           1/*X*/		// DMA request ID for APB SPI1 RX


// APB SPI
static APB_SPI_INFO apb_spi1_info = {0};

#if (APB_SPI1_DMA_EN == 1)
void apb_spi1_dma_tx_event(unsigned int event);
static APB_SPI_DMA apb_spi1_dma_tx =
{
	APB_SPI1_DMA_TX_CH,
	APB_SPI1_DMA_TX_REQID,
	apb_spi1_dma_tx_event
};

void apb_spi1_dma_rx_event(unsigned int event);
static APB_SPI_DMA apb_spi1_dma_rx =
{
	APB_SPI1_DMA_RX_CH,
	APB_SPI1_DMA_RX_REQID,
	apb_spi1_dma_rx_event
};
#endif

// APB SPI resources
static const APB_SPI_RESOURCES apb_spi1_resources =
{
	{0},
	APB_SPI1,
	IRQ_APB_SPI1_SOURCE,
#if (APB_SPI1_DMA_EN == 1)
	&apb_spi1_dma_tx,
	&apb_spi1_dma_rx,
#endif
	&apb_spi1_info
};


// ** Functions ** //

// Get capabilities
APB_SPI_CAPABILITIES apb_spi1_get_capabilities(void);

// Initializes
int apb_spi1_initialize(APB_SPI_SignalEvent_t cb_event);

// Uninitialized
int apb_spi1_uninitialize (void);

// Power control
int apb_spi1_power_control(APB_SPI_POWER_STATE state);

// Send data
int apb_spi1_send(const void *data, unsigned int num);

// Receive data
int apb_spi1_receive(void *data, unsigned int num);

// Transfer data
int apb_spi1_transfer(const void *data_out, void *data_in, unsigned int num);

// Get data count
unsigned int apb_spi1_get_data_count(void);

// Control command
int apb_spi1_control(unsigned int control, unsigned int arg);

// Get status
APB_SPI_STATUS apb_spi1_get_status(void);

#if (APB_SPI1_DMA_EN == 1)
// DMA TX event
void apb_spi1_dma_tx_event (unsigned int event);

// DMA RX event
void apb_spi1_dma_rx_event (unsigned int event);
#endif

// APB SPI interrupt handler
void apb_spi1_irq_handler(void);


// As this template, user can define more APB SPI or Flash devices.


#ifdef __cplusplus
}
#endif


#endif	/* __APB_SPI_DEV_H__ */
