/*
 * ******************************************************************************************
 *
 * File		: spi_flash.c
 * Author	: GOWIN
 * Function	: APB SPI Flash demo.
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "spi.h"
#include "apb_spi_flash_driver.h"
#include "apb_spi_dev.h"

#include "delay.h"
#include "uart.h"
#include <stdio.h>
#include <string.h>


// Definitions ------------------------------------------------------------------------------

#define BYTES_NUM 	1055				// Data size
#define DATA_S_ADDR	0x800000			// Data address

unsigned char tx_buffer[BYTES_NUM] = {0};	// Write data
unsigned char rx_buffer[BYTES_NUM] = {0};	// Read data


// Main function entry
void apb_spi_flash_demo(void)
{
	int i = 0, rest = 0;

	// Initializes UART2
	uart_init(38400);			// Baud rate is 38400

	printf("\r\nIt's an APB SPI Flash demo.\r\n\r\n");

	// Initializes APB SPI Flash
	apb_spi_flash_init(APB_SPI_FLASH);

	simple_delay(1000);		// Wait APB SPI Flash stable

	// Switch APB SPI Flash mode
	apb_spi_flash_switch_mode(APB_SPI_FLASH);

	// ********** Write data to APB SPI Flash ********** //
	for(i = 0;i < BYTES_NUM;i++)
	{
		tx_buffer[i] = i % 256;
	}

	printf("Write data to APB SPI Flash...\r\n");

	// Erase APB SPI Flash before to write data
	apb_spi_flash_erase_4ksec(APB_SPI_FLASH, DATA_S_ADDR);

	// Write data
	if(BYTES_NUM <= 256)
	{
		apb_spi_flash_write(APB_SPI_FLASH, BYTES_NUM, APB_SPI_FLASH_PROGRAM_CMD, DATA_S_ADDR, tx_buffer);
	}
	else
	{
		for(i = 0;i < BYTES_NUM/256;i++)
		{
			apb_spi_flash_write(APB_SPI_FLASH, 256, APB_SPI_FLASH_PROGRAM_CMD, DATA_S_ADDR+i*0x100, tx_buffer+i*256);
		}
		rest = BYTES_NUM - i*256;
		if(rest != 0)
		{
			apb_spi_flash_write(APB_SPI_FLASH, rest, APB_SPI_FLASH_PROGRAM_CMD, DATA_S_ADDR+i*0x100, tx_buffer+i*256);
		}
	}

	printf("Write data done.\r\n\r\n");

	// ********** Read data from APB SPI Flash ********** //
	printf("Read data from APB SPI Flash...\r\n");

	// Read data
	if(BYTES_NUM <= 256)
	{
		apb_spi_flash_read(APB_SPI_FLASH, BYTES_NUM, APB_SPI_FLASH_READ_CMD, DATA_S_ADDR, rx_buffer);
	}
	else
	{
		for(i = 0;i < BYTES_NUM/256;i++)
		{
			apb_spi_flash_read(APB_SPI_FLASH, 256, APB_SPI_FLASH_READ_CMD, DATA_S_ADDR+i*0x100, rx_buffer+i*256);
		}
		rest = BYTES_NUM - i*256;
		if(rest != 0)
		{
			apb_spi_flash_read(APB_SPI_FLASH, rest, APB_SPI_FLASH_READ_CMD, DATA_S_ADDR+i*0x100, rx_buffer+i*256);
		}
	}

	// Output data
	for(i = 0; i < BYTES_NUM; i++)
	{
		printf("%d ", rx_buffer[i]);

		if(((i+1)%256) == 0)
		{
			printf("\r\n");
		}
	}
	printf("Read data done.\r\n");

	// Compared
	if(memcmp(rx_buffer, tx_buffer, BYTES_NUM) == 0)
	{
		printf("\r\nAPB SPI Flash R/W PASS.\r\n");
	}
	else
	{
		printf("\r\nAPB Flash R/W FAIL.\r\n");
	}
}
